## Copyright (C) 2000, 2001, 2002, 2003, 2004  SWsoft
# All rights reserved.
#
# Error/Warning reporting system for pamethod plugins

package HSPC::MT::PM::Tools;

use strict;
##use Exporter;

use SWsoft::protect;


use HSPC::Logger qw(sw_atrace sw_log sw_die);
use HSPC::WebDB qw(:DEFAULT atrans);
use HSPC::Localization qw(string);

use HSPC::DataRequest;
use HSPC::MT::PM::Constants qw(SW_PM_STATUS_DELETED SW_PM_STATUS_OK SW_PM_STATUS_UNCHECKED);

use HSPC::MT::PM::PluginEngine;
use HSPC::MT::PluginConfig;
use HSPC::MT::PluginConfig::PP;

##use HSPC::MT::Core;
##use HSPC::MT::Core::Constants;

##use HSPC::MT::PP::Constants;
##use HSPC::MT::PP;


my $__PM_TEMPLATES;

sub init {
	my $class = shift;

	## Init an empty array
	$__PM_TEMPLATES = [];

	my $templates = HSPC::MT::PluginConfig->find_templates(
		plugin_type      => 'PM',
		force_permission => 1,
	);

	foreach my $template_id (@$templates) {
		my $template = HSPC::MT::PluginConfig->find_mt_template(
			plugin_type => 'PM',
			template_id => $template_id,
		);
		next unless $template;

		eval "use $template;";
		if ($@) {
			sw_log_warn(__PACKAGE__ . "Can not load template $template: $@");
			next;
		}

		if ($template->can('provided_payment_method_types')) {
			my $plugin_type = $class->get_plugin_type(template_id => $template_id);
			foreach my $type (@{$template->provided_payment_method_types}) {
				push(@$__PM_TEMPLATES,{
					template_id   => $template_id,
					pm_title      => $template->title_id,
					plugin_type   => $plugin_type,
					type          => $type->{type},
					type_title_id => $type->{title_id},
					image_id      => $type->{image_id},
				});
			}
		} else {
			sw_log_warn(__PACKAGE__ . "Template $template does not have the mandatory provided_payment_method_types method");
			next;
		}
	}

	return undef;
}

sub get_pm_template_id_by_type {
	my $class = shift;
	my %h = (
		type    => undef,
		@_
	);
	my ($template_id) = map { $_->{template_id} } grep
		{ $_->{type} eq $h{type} } @$__PM_TEMPLATES;

	return $template_id;
}

## Description:
##	Returns information on requested payment method type(s)
## Usage:
##	$res = HSPC::MT::PM::Tools->get_pm_template_description(
##		template_id => 'OP_CCard',
##		plugin_type => 'OP',
##		template_group => 1 - Group by template_id
##	); 
## Parameters:
##	>> class
##	=> template_id
##	=> plugin_type
##	=> template_group
## Returns:
##	reference to an array of hashes
##	{type =>, type_title_id =>, image_id =>, template_id =>, pm_title =>, plugin_type =>}
sub get_pm_template_description {
	my $class = shift;
	my %h = (
		template_id    => undef,
		plugin_type    => undef,
		template_group => undef,
		types          => undef,
		@_
	);

	my @result;
	if ($h{plugin_type}) {
		@result = grep { lc($_->{plugin_type}) eq lc($h{plugin_type}) } @$__PM_TEMPLATES;
	} elsif ($h{template_id}) {
		@result = grep { lc($_->{template_id}) eq lc($h{template_id}) } @$__PM_TEMPLATES;
	} elsif ($h{types} && ref($h{types}) eq 'ARRAY') {
		foreach my $type (@{$h{types}}) {
			push(@result, grep { $_->{type} eq $type } @$__PM_TEMPLATES);
		}
	} else {
		push(@result, @$__PM_TEMPLATES) if ( scalar @$__PM_TEMPLATES );
	}

	return \@result;
}

## usage: $class->get_paymethod_types_suppurted_by_plugin( plugin_id => 'protx' );
## plugin_id =>>
## < list of supported paymethod types
sub get_paytypes_supported_by_plugin {
	my $class = shift;
	my %h = (
		id => undef,
		@_
	);

	my $trace = sw_atrace("id=>$h{id}");

	my $id = $h{id} || sw_die("Specify id, please");

	my $paytype_rows = select_hashrows( 
		q{
			SELECT
				config_id,
				type
			FROM
				plugin_pp_paytypes
			WHERE
				config_id = ?
		},
		$id
	) || [];

	my @paytypes = map { $_->{type} } @$paytype_rows;

	$trace->add("found ".scalar(@paytypes)." paytypes");
	return \@paytypes;
}

## usage: $class->get_plugins_support_paytype( paytype => 'paytype' );
## paytype =>>
## < list of plugins that support paymethod type
sub get_plugins_supported_by_paytype {
	my $class = shift;
	my %h = (
		type      => undef,
		vendor_id => undef,
		@_
	);

	my $trace = sw_atrace("type=>$h{type}");

	sw_die("Specify type and vendor_id, please") unless $h{type} || $h{vendor_id};

	my $plugin_rows = select_hashrows(q{
			SELECT
				type.config_id
			FROM
				plugin_pp_paytypes type,
				plugin
			WHERE
				type.config_id = plugin.id AND
				is_active=1 AND
				type.type = ? AND
				plugin.vendor_id = ?
		}, $h{type}, $h{vendor_id}
	) || [];

	my @plugins = map {$_->{config_id}} @$plugin_rows;

	$trace->add("found ".scalar(@plugins)." plugins");
	return \@plugins;
}

############################################################################
## Method plugin type extracted from the provided template_id
## Parameters:
##   =>> template_id
## Return value:
##   << string represents plugin type
##
sub get_plugin_type {
	my $class = shift;
	my %h = (
		template_id => undef,
		@_
	);

	my $trace = sw_atrace("template_id=$h{template_id}");

	$h{template_id} =~ /(\w+)_[\w\d]+/;

	my $plugin_type = lc($1);
	$trace->add($plugin_type);
	return $plugin_type;
}

############################################################################
## Method finds full parent presentation class name by plugin_type
## Parameters:
##   =>> plugin_type
## Return value:
##   << string contains full class name
##
sub get_presentation_engine {
	my $class = shift;
	my %h = (
		plugin_type => undef,
		template_id => undef,
		@_
	);

	my $trace = sw_atrace("plugin_type=$h{plugin_type}, template_id=$h{template_id}");
	if ($h{template_id}) {
		$h{plugin_type} = $class->get_plugin_type(
			template_id => $h{template_id},
		);
	}

	my %map_subtype = (
		'op' => '::OnlineAccount',
		'bt' => '::BankAccount',
	);

	my $engine = 'HSPC::PM::PluginEngine' . $map_subtype{lc($h{plugin_type})};

	$trace->add($engine);
	return $engine;
}

############################################################################
## Method finds full parent presentation class name by plugin_type
## Parameters:
##   =>> plugin_type
## Return value:
##   << string contains full class name
##
sub get_mt_engine {
	my $class = shift;
	my %h = (
		plugin_type => undef,
		template_id => undef,
		@_
	);

	my $trace = sw_atrace("plugin_type=$h{plugin_type}, template_id=$h{template_id}");
	if ($h{template_id}) {
		$h{plugin_type} = $class->get_plugin_type(
			template_id => $h{template_id},
		);
	}

	my %map_subtype = (
		'op' => '::OnlineAccount',
		'bt' => '::BankAccount',
	);

	my $engine = 'HSPC::MT::PM::PluginEngine' . $map_subtype{lc($h{plugin_type})};

	$trace->add($engine);
	return $engine;
}

sub pm_allowed_types {
	my $class = shift;
	my %h = (
		template_id => undef,
		vendor_id   => undef,
		plugin_id   => undef,
		@_
	);

	my $trace = sw_atrace("template_id=$h{template_id}, plugin_id=$h{plugin_id}, vendor_id=$h{vendor_id}");

	my $plugin_type = $class->get_plugin_type(
		template_id => $h{template_id}
	);

	my $allowed_types = [];

	my $data = HSPC::MT::PP->get_pp_plugin_info(
		vendor_id => $h{vendor_id},
		is_active => 1,
		is_direct => 1,
		sub_type  => uc($plugin_type),
		plugin_ids => $h{plugin_id} ? [$h{plugin_id}] : undef,
	);

	if (@$data) {
		my %types;
		foreach my $plugin_info (@$data) {
			my $plugin = HSPC::MT::PluginConfig::PP->find_plugin(
				id        => $plugin_info->{id},
				vendor_id => $h{vendor_id},
			);

			foreach my $type (@{$plugin->paymethod_types}) {
				$types{$type->{type}}=$type unless $types{$type->{type}};
			}
		}

		@$allowed_types = map {[$_->{type}, string($_->{type_title_id})]} values %types;
	}

	$trace->add('found ' . (scalar @$allowed_types) . ' types');
	return $allowed_types;
}

## usage $class->delete_paymethods_due_to_key_droped
## rec_ids =>> a reference to an array with rec_ids
sub delete_paymethods_due_to_key_droped {
	my $class = shift;
	my %h = (
		rec_ids => undef,
		@_
	);

	my $trace = sw_atrace("got " . scalar @{$h{rec_ids}} . " to delete");

	my $rec_ids = $h{rec_ids};

	if ( $rec_ids && ref $rec_ids eq 'ARRAY' && scalar @$rec_ids ) {
		select_run(
			"UPDATE plugin_pm SET status='" . &SW_PM_STATUS_DELETED .
			"', data_id=0, recurring=0, enabled=0 WHERE data_id IN ('" . 
			join("','", @$rec_ids) . "')"
		);
	}

	return undef;
};

## Description:
##	This method returns a list of valid paymethods for account.
##	Can be overriden in descendants.
## Parameters:
##	>>  class
##	=>> account_no
##	=>  reguest
##	    (only 'OK' otherwise) 
## Returns:
##	data of processed request object
sub get_active_paymethods {
    my $class = shift;
    my %h = (
		account_no  => undef,
		request     => undef, 
		@_
	);
	my $request    = $h{request};
	my $account_no = $h{account_no} || sw_die('account_no undefined');

	if (!$request) {
		$request = HSPC::DataRequest->new(cols => [qw(id)]);
	}

	$request->set_where(
		'status' => [&SW_PM_STATUS_OK,&SW_PM_STATUS_UNCHECKED]
	); 
	$request->set_where('account_no' => $account_no);
    $request->set_where('enabled' => 1);
    HSPC::MT::PM::PluginEngine->load_data(request => $request);

    return $request->{data};
}

## Description:
##	This method returns a list of available paymethod plugins.
##	It should be used in UI for lists of paymethods showed in the store.
## Usage:
##	HSPC::MT::PM::Tools->find_paymethod_plugins(plugin_type => $plugin_type);
## Parameters:
##	>>  class
##  =>  plugin_type
##  =>  active
##  =>  vendor_id
## Returns:
##	array of template_id or []
sub find_paymethod_plugins {
	my $class = shift;
	my %h = (
		plugin_type => undef,
		active      => undef,
		vendor_id   => undef,
		@_
	);

	my $trace = sw_atrace();

	my $sql = q{
		SELECT
			DISTINCT(type.template_id) template_id
		FROM
			plugin_pp_paytypes type,
			plugin,
			plugin_pp pp
		WHERE
			plugin.plugin_type='PP' AND
			plugin.id = type.config_id AND
			pp.is_direct=1 AND
			pp.config_id=plugin.id
	};

	if ($h{plugin_type}) {
		$sql .= " AND LCASE(LEFT(plugin.template_id,INSTR(plugin.template_id, '_')-1)) = '$h{plugin_type}'";
	}
	if ($h{active}) {
		sw_die(__PACKAGE__. 'The vendor_id parameter must be provided if active=1')
			unless ($h{vendor_id});
		$sql .= " AND plugin.is_active = 1 AND plugin.vendor_id = $h{vendor_id}"
	}
	$sql .= ' ORDER BY plugin.template_id desc';
	
	my $rows = select_hashrows( $sql ) || [];

	my @templates = map { $_->{template_id} } @$rows;

	$trace->add("Found " . (scalar @templates) . " templates");

	return scalar @templates ? \@templates : [];
}

## Description:
##	This method check if vendor keys stored in database.
## Usage:
##	$this->get_vendor_skey(vendor_id => $id);
## Parameters:
##	>>  class
##	=>> vendor_id
## Returns:
##	key_id or undef 
sub get_vendor_skey {
	my $class = shift;
	my %h     = (
		vendor_id => undef,
		@_
	);
	my $trace = sw_atrace("vendor_id=>$h{vendor_id}");

	sw_die("[HSPC::MT::CC] check_vendor_skey(): vendor_id not specified")
		unless defined($h{vendor_id});

	my $pr_config = $class->get_vendor_config( vendor_id => $h{vendor_id} );

	my $key_id =
		($pr_config->key1) ? $pr_config->key1
		: ($pr_config->key2) ? $pr_config->key2
			: undef;

	my $key_id2;
	if (($key_id) && ($key_id == $pr_config->key1)) {
		$key_id2 = ($pr_config->key2) ? $pr_config->key2 : undef;
	}

	unless ($key_id) {
		$trace->add("no key found");
		return undef;
	}

	## check if we store secure part for keys in database
	my $is_key_stored = HSPC::MT::SS->is_skey_stored(key_id => $key_id);

	## check second key if check failed for the first one
	if ((!$is_key_stored) && ($key_id2)) {
		$is_key_stored = HSPC::MT::SS->is_skey_stored(key_id => $key_id2);
		$key_id = $key_id2;
	}

	$trace->add($is_key_stored ? "key_id=>$key_id" : "not stored");
	return ($is_key_stored) ? $key_id : undef;
}

## Description:
##	Load ProviderConfig object for vendor of account_no.
## Returns:
##	ProviderConfig object.
sub get_vendor_config {
	my $class = shift;

	my %h = (
		vendor_id => undef,
		account_no => undef,
		@_
	);

	my $vendor_id = $h{vendor_id};
	my $account_no = $h{account_no};

	unless ( $account_no || $vendor_id ) {
		sw_die(__PACKAGE__ . "Either account_no or vendor_id parameter must be provided");
	}

	if ( $account_no ) {
		$vendor_id = HSPC::MT::Core->find_account_vendor_id($account_no);
	}

	unless ($vendor_id) {
		sw_die(__PACKAGE__ . "Can not load vendor config. Vendor ID is undefined.");
	}

	my $pr_config = HSPC::MT::GM::ProviderConfig->new();
	$pr_config->load(id=>$vendor_id);

	return ($pr_config && $pr_config->id) ? $pr_config : undef;
}












##-------------- OLD METHODS ----------------------------------


## usage $class->set_supported_paytypes_for_plugin
## plugin_id =>>
## paytypes =>> list of paytypes
sub set_paytypes_supported_by_plugin {
	my $class = shift;
	my %h = (
		plugin_id => undef,
		paytypes  => undef,
		@_
	);

	my $trace = sw_atrace("plugin_id=>$h{plugin_id}");

	my $plugin_id = $h{plugin_id} || sw_die("Specify plugin_id, please");
	my $paytypes  = $h{paytypes}  || sw_die("Specify paytypes list, please");

	my $trans = atrans();
	{
		select_run("DELETE FROM pp_supported_paytypes WHERE plugin_id = ?", $plugin_id);

		foreach my $paytype (@$paytypes) {
			select_run(
				q{
				INSERT INTO
					pp_supported_paytypes
				(plugin_id, paytype)
				VALUES (?, ?)
			},
			$plugin_id, $paytype);
		}
	}
	$trans->commit();
};

## usage $class->delete_supported_paytypes_for_plugin
## plugin_id =>>
sub delete_paytypes_supported_by_plugin {
	my $class = shift;
	my %h = (
		plugin_id => undef,
		@_
	);

	my $trace = sw_atrace("plugin_id=>$h{plugin_id}");

	my $plugin_id = $h{plugin_id} || sw_die("Specify plugin_id, please");

	select_run("DELETE FROM pp_supported_paytypes WHERE plugin_id = ?", $plugin_id);

};

sub install_paymethod_plugin {
	my $this = shift;
	my %h = @_;

	my $id                      = $h{id}                      || sw_die('id undefined');
	my $cath_id                 = $h{cath_id}                 || $id;
	my $img_id                  = $h{img_id}                  || 'ba_dummy_img';
	my $title                   = $h{title}                   || sw_die('title undefined');
	my $title_id                = $h{title_id}                || sw_die('title_id undefined');
	my $plugin_type             = $h{plugin_type}             || sw_die('plugin_type undefined');
	my $class_name              = $h{class_name}              || sw_die('class_name undefined');
	my $em_class_name           = $h{em_class_name}           || sw_die('em_class_name undefined');
	my $private_class_name      = $h{private_class_name}      || sw_die('private_class_name undefined');
	my $presentation_class_name = $h{presentation_class_name} || sw_die('presentation_class_name undefined');
	my $parent_presentation_class_name = $h{parent_presentation_class_name}
			|| sw_die('parent_presentation_class_name undefined');

	select_run(q|DELETE FROM paymethod_type WHERE id=?|, $id);
	select_run(
		q|
		INSERT INTO paymethod_type (
			id, plugin_id, plugin_type, title, img_id, class_name, presentation_class_name,
			private_class_name, parent_presentation_class_name, em_class_name
		)
		VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?
		)|, $id, $cath_id, $plugin_type, $title, $img_id, $class_name,
		$presentation_class_name, $private_class_name,
		$parent_presentation_class_name, $em_class_name
	);

	select_run(q|REPLACE INTO paymethod_plugin	(id, plugin_type, title_id)
		VALUES (?, ?, ?)|, $cath_id, $plugin_type, $title_id
	);

	return undef;
}


############################################################################
## Method deletes paymethod plugin information from plugin 
## and vendor plugin tables
## Parameters:
##   id                      =>> plugin id
sub uninstall_paymethod_plugin {
	my $this = shift;
	my %h    = @_;
	my $id = $h{id} || sw_die("id undefined");

	if(is_table_exists(table => "paymethod_plugin")) {
		select_run("DELETE FROM paymethod_plugin	WHERE id='$id'");
	}
	if(is_table_exists(table => "paymethod_type")) {
		select_run(q|DELETE FROM paymethod_type WHERE id=?|, $id);
	}

	return undef;
}

1;
